/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_ALGORITHM_H__
#define __TK_ALGORITHM_H__

#include <list>
#include <map>

#include "libtkcommon/numerical-types.h"
#include "libtkcommon/string.h"
#include "libtkrandom/randomness.h"

// #include "liblanczos/matrix.h"
// #include "liblanczos/complex.h"

namespace Tk {
  namespace Algorithm {
    
    typedef short Offset;
    typedef Rational Probability;
  
    struct OffsetDetails
    {
      Offset       offset;
      unsigned     weight;
      Probability  probability;
      OffsetDetails(Offset o, unsigned w, Probability p = Probability())
	: offset(o), weight(w), probability(p) {}
    };
  
    class Offsets : public std::list<OffsetDetails>
    {
    public:
      Offsets() {}
      const OffsetDetails & operator[] (const Offset &offset) const;
    };

    class Algorithm
    {
    private:
    
      Offsets             _offsets;
      SmallNumbers        _outcome_pattern;
      unsigned            _outcome_count;
    
    public:
      /** The spec is a space-seperated list of integers.
       * It defines the probability weights that will be placed
       * on the different possible trace movements.  The
       * list contains (N/2)+1 elements, where N is the number of
       * possibile positions; the last element is the central
       * position.
       *
       * Eg, For an equally distributed probability, one pixel
       * in either direction, the spec would be as follows:
       *
       *  '1 1'
       *
       * which would result in the following distribution:
       *
       *      ---
       *     |XXX|  <-- previous line's pixel
       *  ---|---|---
       * |33%|33%|33%|
       *  --- --- ---
       *
       * The middle pixel's probability is given as the last
       * weight in the spec.
       */
      Algorithm (const String &spec);
    
      Offset choose_offset(unsigned choice) const;
    
      const Offsets & get_offsets() const;
      const SmallNumbers & get_outcome_pattern() const;
      unsigned get_outcome_count() const;
    };

  }; /* namespace Algorithm */
}; /* namespace Tk */

#endif /* __TK_ALGORITHM_H__ */

