/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_DB_CONNECTION_H__
#define __TK_DB_CONNECTION_H__

#include <odbc++/preparedstatement.h>
#include <glibmm.h>

#include "libtkdb/types.h"

namespace Tk {
  namespace Db {
    
    class Connection
      {
      public:
	typedef Glib::RecMutex Mutex;
      private:
	odbc::Connection    *_connection;
	Mutex                _mutex;
      public:
	Connection (const Tk::String &data_source_name = Tk::String("TkResolver++"),
		    const Tk::String &username = Tk::String("psi"),
		    const Tk::String &password = Tk::String("isp"));
	~Connection();
	
	// This must always be locked before calling the methods below
	// and released when their returned values are finished with
	Mutex &get_mutex();
	
	Statement  * new_statement();
	PStatement * prepare_statement(const Tk::String &sql);
	void         execute_statement(const Tk::String &sql);
	DataSet    * execute_query(const Tk::String &sql);
	Id           get_last_insert_id();
	PStatement * get_table_insert_statement(const Tk::String &table, const Tk::Strings &columns);
    };

  }; /* namespace Db */
}; /* namespace Tk */

#endif /* __TK_DB_CONNECTION_H__ */
