/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2008, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libtkdbui/property_table.h"
#include "libtkdbui/property.h"
#include "libtkdb/exception.h"

using namespace Tk::Db::Ui;

PropertyTable:: PropertyTable(Tk::Db::Database *database,
			      const Tk::String &domain,
			      const Tk::Db::Id &entity_id)
  : Gtk::Table(1, 2)
{
  Tk::Db::Data::TypeMap domain_property_types(database->get_domain_property_types(domain));
  
  resize(domain_property_types.size(), 2);
  set_col_spacings(5);
  
  unsigned i(0);
  for (Tk::Db::Data::TypeMap::iterator prop_iter( domain_property_types.begin() );
       prop_iter != domain_property_types.end();
       ++prop_iter, ++i)
    {
      Gtk::Label *label = Gtk::manage( new Gtk::Label(prop_iter->first) );
      attach(*label, 0, 1, i, i+1, Gtk::FILL, static_cast<Gtk::AttachOptions>(0));
      
      Property *prop;
      try
	{
	  Tk::Db::Data::Value value( database->get_property(domain, entity_id, prop_iter->first) );
	  prop = Gtk::manage( new Property(database, domain, entity_id, prop_iter->first, value) );
	}
      catch (Tk::Db::PropertyNotSet &pns)
	{
	  prop = Gtk::manage( new Property(database, domain, entity_id, prop_iter->first, prop_iter->second) );
	}
      
      attach(*prop, 1, 2, i, i+1, Gtk::FILL|Gtk::EXPAND, Gtk::FILL);
    }
}

