/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_RANDOM_RETRIEVER_REGISTRY_H__
#define __TK_RANDOM_RETRIEVER_REGISTRY_H__

#include "libtkcommon/string.h"
#include "retriever.h"

namespace Tk {

  namespace Random {

    class RetrieverType
    {
    public:
      virtual bool is_true_randomness() = 0;
      virtual Retriever * new_retriever(unsigned count) = 0;
    };

    typedef std::pair<RetrieverType *, String> RetrieverDetails;
    typedef std::map<String, RetrieverDetails> RetrieverTypeMap;

    class RetrieverRegistry
    {
    private:
      static const RetrieverTypeMap _registry;
      
      static const RetrieverDetails & get_retriever_details(const String &type);
      static RetrieverType * get_retriever (const String &type);
    public:
      static Strings get_names();
      static bool is_true_randomness(const String &type);
      static Retriever * new_retriever(const String &name, unsigned count = 0);
      static const String & get_retriever_description(const String &name);
    };

  };
  
};


#endif /* __TK_RANDOM_RETRIEVER_REGISTRY_H__ */
