/* -*- c++ -*-
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2005, 2006, 2007, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef __TK_RESOLVER_RESOLVER_H__
#define __TK_RESOLVER_RESOLVER_H__

#include "libtkcommon/string.h"
#include "libtkcommon/value.h"
#include "libtkalgorithm/algorithm.h"
#include "libtkrandom/decision_tree.h"
#include "libtkrandom/randomness.h"
#include "libtkresolver/display.h"
#include "libtkresolver/types.h"

namespace Tk {
  namespace Resolver {

    class Resolver
    {
    private: 
      typedef Tk::Algorithm::Algorithm Algorithm;
      
      class Drawer : public Tk::Random::Effector
      {
      private:
	typedef Tk::Random::Choice Choice;
	typedef std::list<Choice> Choices;
	typedef std::vector<ColourGenerator> ColourGenerators;
	
	const Algorithm      _algorithm;
	const unsigned       _trace_count;
	const bool           _test;
	
	Display              _display;
	Time                 _actual_time;
	
	ColourGenerators    *_colours;
	std::list<Choice>    _choices;
	Result              *_result;
	
      public:
	Drawer(const Algorithm &algorithm,
	       bool coloured,
	       unsigned height,
	       unsigned width,
	       unsigned trace_count,
	       const DirectionValue &arrow,
	       unsigned total_time,
	       double horizon,
	       bool test);
	~Drawer();
	
	void start();
	virtual bool affect(Tk::Random::Choice choice);
	
	unsigned get_display_width() const;
	unsigned get_display_height() const;
 	const Time & get_actual_time() const;
	Result *take_result();
      };
      
      Algorithm                _algorithm;
      Tk::Random::DecisionTree _decision_tree;

    public:
      Resolver(const String &algo, unsigned bit_multiplier);

      Result *run(Tk::Random::Retriever &retriever,
		  Tk::Random::Randomness &randomness,
		  bool coloured,
		  unsigned &height,
		  unsigned &width,
		  unsigned trace_count,
		  const DirectionValue &arrow,
		  unsigned total_time,
		  double horizon,
		  bool test,
		  unsigned &retries,
		  Time &actual_time);
    };
    
    typedef std::auto_ptr<Resolver> ResolverPtr;
    
  }; /* namespace Resolver */
}; /* namespace Tk */

#endif /* __TK_RESOLVER_RESOLVER_H__ */
