/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004, 2009 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include <iostream>
#include <stdexcept>
#include <string>

#include "sdl.h"

using namespace Tk::Resolver;

bool Tk::Resolver::SDL::_initialised(false);

SDL:: SDL()
{
  initialise();
}

void
SDL::initialise()
{
  if (_initialised)
    return;
  
  int err = SDL_Init(SDL_INIT_VIDEO|SDL_INIT_TIMER|SDL_INIT_NOPARACHUTE);
  if (err)
    {
      std::cerr << "could not initialise SDL: "
		<< SDL_GetError()
		<< std::endl;
      throw std::runtime_error(std::string("could not initialise SDL: ") + SDL_GetError());
    }

  atexit(SDL_Quit);
  
  _initialised = true;
}

void
SDL:: reset()
{
  SDL_QuitSubSystem(SDL_INIT_VIDEO);
  
  int err = SDL_InitSubSystem(SDL_INIT_VIDEO);
  if (err)
    throw std::runtime_error(std::string("could not reinitialise SDL video: ") + SDL_GetError());
}

bool
SDL:: check_escape(bool test)
{
  SDL_Event event;

  while (SDL_PollEvent(&event))
    {
      switch (event.type)
	{
	case SDL_QUIT:
	  return true;

	case SDL_KEYUP:
	  switch (event.key.keysym.sym)
	    {
	    case SDLK_ESCAPE:
	      if (test)
		return true;
	      break;

	    default:
	      break;
	    }
	  
	default:
	  break;
	}
      
    }

  return false;
}

void
SDL:: lock(SDL_Surface *surface)
{
  if (SDL_MUSTLOCK(surface))
    {
      int err( SDL_LockSurface(surface) );
      if (err)
	throw std::runtime_error(std::string("could not lock SDL surface: ") + SDL_GetError());
    }
}

void
SDL:: unlock(SDL_Surface *surface)
{
  if (SDL_MUSTLOCK(surface))
    SDL_UnlockSurface(surface);
}

