/*
 * TkResolver++ - Psychokinesis testing software
 *
 * Copyright (C) 2004 Robert Ham <rah@bash.sh>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#ifndef _GNU_SOURCE
#define _GNU_SOURCE
#endif

#include "libtkcommon/debug.h"
#include "libtkcommon/numerical-types.h"
#include "libtkrandom/decision_tree.h"
#include "libtkrandom/retriever_registry.h"

using namespace Tk::Random;

typedef std::map<Choice, unsigned long> ChoiceCounts;

class Tester : public Tk::Random::Effector
{
public:
  const unsigned _test_count;
  ChoiceCounts _choice_counts;
  unsigned long _total_count;
  Tester(unsigned test_count)
    : _test_count(test_count),
      _total_count(0)
  {}
  virtual bool affect(Choice choice)
  {
    ++_choice_counts[choice];
    ++_total_count;
    bool cont(_total_count < _test_count);
    /*
    WDEBUG_OUT << L"Choice "
	       << _total_count
	       << L" of "
	       << _test_count
	       << L": "
	       << choice
	       << L" ("
	       << (cont ? "continuing" : "not continuing")
	       << L")"
	       << std::endl;
    */
    return cont;
  }
};

void
test_decision_output(Retriever &retriever, unsigned noutcomes, unsigned bit_multiplier, unsigned test_count)
{
  DecisionTree dt(noutcomes, bit_multiplier);
  Randomness r;
  
  Tester tester(test_count);
  dt.decide(retriever, r, tester);
  
  std::wcout << L"Outcomes: "
	     << noutcomes
	     << L"; bits: "
	     << bit_multiplier
	     << L"; tests: "
	     << test_count
	     << std::endl
	     << L"  ";
  for (ChoiceCounts::iterator iter( tester._choice_counts.begin() );
       iter != tester._choice_counts.end();
       ++iter)
    {
      if (iter != tester._choice_counts.begin())
	std::wcout << L"; ";
      
      Tk::Real ratio(Tk::Real(iter->second) / Tk::Real(tester._total_count));
      std::wcout << iter->first
		 << L": "
		 << iter->second
		 << L" ("
		 << Tk::String(ratio)
		 << L")" ;
    }
  std::wcout << std::endl;
}

int
main (int argc, char ** argv)
{
  Retriever *retriever( RetrieverRegistry::new_retriever("alea") );
  
  test_decision_output(*retriever, 2, 1, 10000);
  test_decision_output(*retriever, 3, 1, 10000);
  test_decision_output(*retriever, 5, 1, 10000);
  test_decision_output(*retriever, 7, 1, 10000);
  test_decision_output(*retriever, 2, 1, 1000000);
  test_decision_output(*retriever, 3, 1, 1000000);
  test_decision_output(*retriever, 5, 1, 1000000);
  test_decision_output(*retriever, 7, 1, 1000000);
  
  delete retriever;
}
